<?php

namespace Drupal\tmgmt_contentapi\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Process\Exception\RuntimeException;

/**
 * Controller for processing the queue.
 */
class QueueProcessController extends ControllerBase {

  /**
   * Process the queue in batches.
   */
  public function processQueueInBackground($queue_name, int $batch_size = 100) {
    $queue = \Drupal::service('queue')->get($queue_name);
    $queue_worker = \Drupal::service('plugin.manager.queue_worker')->createInstance($queue_name);
    $processed = 1;
    \Drupal::logger('TMGMT_CONTENTAPI')->notice('Queue processing started in background: @queue', ['@queue' => $queue_name]);

    // Check if any item exist in queue, if not then return.
    if (!$queue->numberOfItems()) {
      \Drupal::logger('TMGMT_CONTENTAPI')->notice('No Items to process in queue: @queue', ['@queue' => $queue_name]);
      return new Response('No item found to process');
    }

    // Process elements in a batch.
    while ($processed <= $batch_size && ($item = $queue->claimItem())) {
      try {
        // Process the item.
        $queue_worker->processItem($item->data);
        // Delete the item from the queue.
        $queue->deleteItem($item);
      }
      catch (\Exception $e) {
        \Drupal::logger('TMGMT_CONTENTAPI')->error('Error processing task: @message', ['@message' => $e->getMessage()]);
        // Optionally release the item for retry.
        $queue->releaseItem($item);
      }
      // Unset variables to free memory.
      unset($item);
      // Reset cache after processing each item.
      \Drupal::service('entity_type.manager')->getStorage('node')->resetCache();
      drupal_static_reset();
      // Collect garbage after processing each item.
      gc_collect_cycles();
      // Increment processed count.
      $processed++;
    }
    unset($queue_worker);
    unset($queue);
    return new Response('Processed ' . $processed . ' items.');
  }

}
